@@ -15,6 +15,10 @@ static bool should_keep(const InputSection<E> &isec) {
   u32 flags = isec.shdr().sh_flags;
   std::string_view name = isec.name();
 
+  if constexpr (is_ppc32<E>)
+    if (name == ".got2")
+      return true;
+
   return (flags & SHF_GNU_RETAIN) ||
          type == SHT_NOTE ||
          type == SHT_INIT_ARRAY ||
@@ -114,8 +118,6 @@ static void collect_root_set(Context<E> &ctx,
   });
 
   // Add sections referenced by root symbols.
-  enqueue_symbol(get_symbol(ctx, ctx.arg.entry));
-
   for (Symbol<E> *sym : ctx.arg.undefined)
     enqueue_symbol(sym);
 
