@@ -95,12 +95,12 @@ struct SymbolAux<PPC64V1> : SymbolAux<X86_64> {
 //
 
 template <typename E>
-class RangeExtensionThunk {};
+class Thunk {};
 
 template <needs_thunk E>
-class RangeExtensionThunk<E> {
+class Thunk<E> {
 public:
-  RangeExtensionThunk(OutputSection<E> &osec, i64 offset)
+  Thunk(OutputSection<E> &osec, i64 offset)
     : output_section(osec), offset(offset) {}
 
   i64 size() const { return E::thunk_hdr_size + symbols.size() * E::thunk_size; }
@@ -119,7 +119,7 @@ public:
   std::vector<Symbol<E> *> symbols;
 };
 
-struct RangeExtensionRef {
+struct ThunkRef {
   i16 thunk_idx = -1;
   i16 sym_idx = -1;
 };
@@ -212,7 +212,7 @@ struct InputSectionExtras {};
 
 template <needs_thunk E>
 struct InputSectionExtras<E> {
-  std::vector<RangeExtensionRef> range_extn;
+  std::vector<ThunkRef> thunk_refs;
 };
 
 template <is_riscv E>
@@ -340,14 +340,12 @@ void write_plt_entry(Context<E> &ctx, u8 *buf, Symbol<E> &sym);
 template <typename E>
 void write_pltgot_entry(Context<E> &ctx, u8 *buf, Symbol<E> &sym);
 
-typedef enum { HEADER, OUTPUT_SECTION, SYNTHETIC } ChunkKind;
-
 // Chunk represents a contiguous region in an output file.
 template <typename E>
 class Chunk {
 public:
   virtual ~Chunk() = default;
-  virtual ChunkKind kind() { return SYNTHETIC; }
+  virtual bool is_header() { return false; }
   virtual OutputSection<E> *to_osec() { return nullptr; }
   virtual i64 get_reldyn_size(Context<E> &ctx) const { return 0; }
   virtual void construct_relr(Context<E> &ctx) {}
@@ -397,7 +395,7 @@ public:
     this->shdr.sh_addralign = sizeof(Word<E>);
   }
 
-  ChunkKind kind() override { return HEADER; }
+  bool is_header() override { return true; }
   void copy_buf(Context<E> &ctx) override;
 };
 
@@ -411,7 +409,7 @@ public:
     this->shdr.sh_addralign = sizeof(Word<E>);
   }
 
-  ChunkKind kind() override { return HEADER; }
+  bool is_header() override { return true; }
   void copy_buf(Context<E> &ctx) override;
 };
 
@@ -425,7 +423,7 @@ public:
     this->shdr.sh_addralign = sizeof(Word<E>);
   }
 
-  ChunkKind kind() override { return HEADER; }
+  bool is_header() override { return true; }
   void update_shdr(Context<E> &ctx) override;
   void copy_buf(Context<E> &ctx) override;
 
@@ -449,8 +447,11 @@ public:
 template <typename E>
 class OutputSection : public Chunk<E> {
 public:
-  OutputSection(Context<E> &ctx, std::string_view name, u32 type, u64 flags);
-  ChunkKind kind() override { return OUTPUT_SECTION; }
+  OutputSection(std::string_view name, u32 type) {
+    this->name = name;
+    this->shdr.sh_type = type;
+  }
+
   OutputSection<E> *to_osec() override { return this; }
   void construct_relr(Context<E> &ctx) override;
   void copy_buf(Context<E> &ctx) override;
@@ -462,8 +463,9 @@ public:
   void create_range_extension_thunks(Context<E> &ctx);
 
   std::vector<InputSection<E> *> members;
-  std::vector<std::unique_ptr<RangeExtensionThunk<E>>> thunks;
+  std::vector<std::unique_ptr<Thunk<E>>> thunks;
   std::unique_ptr<RelocSection<E>> reloc_sec;
+  Atomic<u32> sh_flags;
 };
 
 template <typename E>
@@ -975,7 +977,7 @@ public:
 
 private:
   ElfChdr<E> chdr = {};
-  std::unique_ptr<Compressor> compressed;
+  std::unique_ptr<Compressor> compressor;
 };
 
 template <typename E>
@@ -1048,11 +1050,8 @@ template <typename E> void write_gdb_index(Context<E> &ctx);
 // discards the other by eliminating all sections that the other
 // comdat section refers to.
 struct ComdatGroup {
-  ComdatGroup() = default;
-  ComdatGroup(const ComdatGroup &other) : owner(other.owner.load()) {}
-
   // The file priority of the owner file of this comdat section.
-  std::atomic_uint32_t owner = -1;
+  Atomic<u32> owner = -1;
 };
 
 template <typename E>
@@ -1160,6 +1159,7 @@ public:
                                std::string archive_name, bool is_in_lib);
 
   void parse(Context<E> &ctx);
+  void initialize_symbols(Context<E> &ctx);
   void initialize_mergeable_sections(Context<E> &ctx);
   void resolve_section_pieces(Context<E> &ctx);
   void resolve_symbols(Context<E> &ctx) override;
@@ -1183,18 +1183,20 @@ public:
   std::vector<FdeRecord<E>> fdes;
   BitVector has_symver;
   std::vector<ComdatGroupRef<E>> comdat_groups;
-  InputSection<E> *eh_frame_section = nullptr;
+  std::vector<InputSection<E> *> eh_frame_sections;
   bool exclude_libs = false;
   std::map<u32, u32> gnu_properties;
-  bool is_lto_obj = false;
   bool needs_executable_stack = false;
+  bool is_lto_obj = false;
+  bool is_gcc_offload_obj = false;
+  bool is_rust_obj = false;
 
-  u64 num_dynrel = 0;
-  u64 reldyn_offset = 0;
+  i64 num_dynrel = 0;
+  i64 reldyn_offset = 0;
 
-  u64 fde_idx = 0;
-  u64 fde_offset = 0;
-  u64 fde_size = 0;
+  i64 fde_idx = 0;
+  i64 fde_offset = 0;
+  i64 fde_size = 0;
 
   // For ICF
   std::unique_ptr<InputSection<E>> llvm_addrsig;
@@ -1205,7 +1207,7 @@ public:
   InputSection<E> *debug_pubtypes = nullptr;
 
   // For LTO
-  std::vector<std::string_view> lto_symbol_versions;
+  std::vector<ElfSym<E>> lto_elf_syms;
 
   // Target-specific member
   [[no_unique_address]] ObjectFileExtras<E> extra;
@@ -1215,7 +1217,6 @@ private:
              std::string archive_name, bool is_in_lib);
 
   void initialize_sections(Context<E> &ctx);
-  void initialize_symbols(Context<E> &ctx);
   void sort_relocations(Context<E> &ctx);
   void initialize_ehframe_sections(Context<E> &ctx);
   void parse_note_gnu_property(Context <E> &ctx, const ElfShdr <E> &shdr);
@@ -1385,11 +1386,13 @@ template <typename E> void compute_section_sizes(Context<E> &);
 template <typename E> void sort_output_sections(Context<E> &);
 template <typename E> void claim_unresolved_symbols(Context<E> &);
 template <typename E> void scan_relocations(Context<E> &);
+template <typename E> void compute_imported_symbol_weakness(Context<E> &);
 template <typename E> void construct_relr(Context<E> &);
 template <typename E> void create_output_symtab(Context<E> &);
 template <typename E> void report_undef_errors(Context<E> &);
 template <typename E> void create_reloc_sections(Context<E> &);
 template <typename E> void copy_chunks(Context<E> &);
+template <typename E> void rewrite_endbr(Context<E> &);
 template <typename E> void apply_version_script(Context<E> &);
 template <typename E> void parse_symbol_version(Context<E> &);
 template <typename E> void compute_import_export(Context<E> &);
@@ -1462,6 +1465,22 @@ public:
 // arch-ppc64v2.cc
 //
 
+extern const std::vector<std::pair<std::string_view, u32>>
+ppc64_save_restore_insns;
+
+class PPC64SaveRestoreSection : public Chunk<PPC64V2> {
+public:
+  PPC64SaveRestoreSection() {
+    this->name = ".save_restore_gprs";
+    this->shdr.sh_type = SHT_PROGBITS;
+    this->shdr.sh_flags = SHF_ALLOC | SHF_EXECINSTR;
+    this->shdr.sh_addralign = 16;
+    this->shdr.sh_size = ppc64_save_restore_insns.size() * 4;
+  }
+
+  void copy_buf(Context<PPC64V2> &ctx) override;
+};
+
 template <> u64 get_eflags(Context<PPC64V2> &ctx);
 
 //
@@ -1586,6 +1605,7 @@ struct ContextExtras<PPC64V1> {
 
 template <>
 struct ContextExtras<PPC64V2> {
+  PPC64SaveRestoreSection *save_restore = nullptr;
   Symbol<PPC64V2> *TOC = nullptr;
   Atomic<bool> is_power10 = false;
 };
@@ -1607,7 +1627,12 @@ struct ContextExtras<ALPHA> {
 // resource management, and other miscellaneous objects.
 template <typename E>
 struct Context {
-  Context() = default;
+  Context() {
+    arg.entry = get_symbol(*this, "_start");
+    arg.fini = get_symbol(*this, "_fini");
+    arg.init = get_symbol(*this, "_init");
+  }
+
   Context(const Context<E> &) = delete;
 
   void checkpoint() {
@@ -1624,6 +1649,9 @@ struct Context {
     CompressKind compress_debug_sections = COMPRESS_NONE;
     SeparateCodeKind z_separate_code = NOSEPARATE_CODE;
     ShuffleSectionsKind shuffle_sections = SHUFFLE_SECTIONS_NONE;
+    Symbol<E> *entry = nullptr;
+    Symbol<E> *fini = nullptr;
+    Symbol<E> *init = nullptr;
     UnresolvedKind unresolved_symbols = UNRESOLVED_ERROR;
     bool Bsymbolic = false;
     bool Bsymbolic_functions = false;
@@ -1650,6 +1678,7 @@ struct Context {
     bool ignore_data_address_equality = false;
     bool is_static = false;
     bool lto_pass2 = false;
+    bool nmagic = false;
     bool noinhibit_exec = false;
     bool oformat_binary = false;
     bool omagic = false;
@@ -1700,6 +1729,7 @@ struct Context {
     bool z_text = false;
     i64 filler = -1;
     i64 spare_dynamic_tags = 5;
+    i64 spare_program_headers = 0;
     i64 thread_count = 0;
     i64 z_stack_size = 0;
     u64 shuffle_sections_seed;
@@ -1711,9 +1741,6 @@ struct Context {
     std::string dependency_file;
     std::string directory;
     std::string dynamic_linker;
-    std::string entry = "_start";
-    std::string fini = "_fini";
-    std::string init = "_init";
     std::string output = "a.out";
     std::string package_metadata;
     std::string plugin;
@@ -1751,11 +1778,11 @@ struct Context {
   bool is_static;
   bool in_lib = false;
   i64 file_priority = 10000;
+  MappedFile<Context<E>> *script_file = nullptr;
   std::unordered_set<std::string_view> visited;
   tbb::task_group tg;
 
   bool has_error = false;
-  bool has_lto_object = false;
   Atomic<bool> has_init_array = false;
   Atomic<bool> has_ctors = false;
 
@@ -1797,7 +1824,7 @@ struct Context {
   Atomic<bool> has_textrel = false;
   Atomic<u32> num_ifunc_dynrels = 0;
 
-  tbb::concurrent_hash_map<std::string_view, std::vector<std::string>> undef_errors;
+  tbb::concurrent_hash_map<Symbol<E> *, std::vector<std::string>> undef_errors;
 
   // Output chunks
   OutputEhdr<E> *ehdr = nullptr;
@@ -2024,6 +2051,10 @@ public:
     TAG_MASK = 0b11,
   };
 
+  static_assert(alignof(InputSection<E>) >= 4);
+  static_assert(alignof(Chunk<E>) >= 4);
+  static_assert(alignof(SectionFragment<E>) >= 4);
+
   uintptr_t origin = 0;
 
   // `value` contains symbol value. If it's an absolute symbol, it is
@@ -2160,30 +2191,27 @@ public:
   // opposed to IR object).
   bool referenced_by_regular_obj : 1 = false;
 
+  // For `-z rewrite-endbr`
+  bool address_taken : 1 = false;
+
   // Target-dependent extra members.
   [[no_unique_address]] SymbolExtras<E> extra;
 };
 
-// If we haven't seen the same `key` before, create a new instance
-// of Symbol and returns it. Otherwise, returns the previously-
-// instantiated object. `key` is usually the same as `name`.
 template <typename E>
 Symbol<E> *get_symbol(Context<E> &ctx, std::string_view key,
-                      std::string_view name) {
-  typename decltype(ctx.symbol_map)::const_accessor acc;
-  ctx.symbol_map.insert(acc, {key, Symbol<E>(name)});
-  return const_cast<Symbol<E> *>(&acc->second);
-}
+                      std::string_view name);
 
 template <typename E>
-Symbol<E> *get_symbol(Context<E> &ctx, std::string_view name) {
-  return get_symbol(ctx, name, name);
-}
+Symbol<E> *get_symbol(Context<E> &ctx, std::string_view name);
+
+template <typename E>
+std::string_view demangle(const Symbol<E> &sym);
 
 template <typename E>
 std::ostream &operator<<(std::ostream &out, const Symbol<E> &sym) {
   if (opt_demangle)
-    out << demangle(sym.name());
+    out << demangle(sym);
   else
     out << sym.name();
   return out;
@@ -2240,7 +2268,7 @@ inline u64 InputSection<E>::get_addr() const {
 template <typename E>
 inline std::string_view InputSection<E>::name() const {
   if (file.elf_sections.size() <= shndx)
-    return ".common";
+    return (shdr().sh_flags & SHF_TLS) ? ".tls_common" : ".common";
   return file.shstrtab.data() + file.elf_sections[shndx].sh_name;
 }
 
@@ -2307,7 +2335,7 @@ InputSection<E>::get_fragment(Context<E> &ctx, const ElfRel<E> &rel) {
 template <typename E>
 u64 InputSection<E>::get_thunk_addr(i64 idx) {
   if constexpr (needs_thunk<E>) {
-    RangeExtensionRef ref = extra.range_extn[idx];
+    ThunkRef ref = extra.thunk_refs[idx];
     assert(ref.thunk_idx != -1);
     return output_section->thunks[ref.thunk_idx]->get_addr(ref.sym_idx);
   }
@@ -2405,8 +2433,6 @@ InputFile<E>::get_string(Context<E> &ctx, const ElfShdr<E> &shdr) {
 
 template <typename E>
 inline std::string_view InputFile<E>::get_string(Context<E> &ctx, i64 idx) {
-  assert(idx < elf_sections.size());
-
   if (elf_sections.size() <= idx)
     Fatal(ctx) << *this << ": invalid section index: " << idx;
   return this->get_string(ctx, elf_sections[idx]);
@@ -2718,9 +2744,6 @@ inline bool Symbol<E>::has_plt(Context<E> &ctx) const {
 
 template <typename E>
 inline bool Symbol<E>::is_absolute() const {
-  if (file && file->is_dso)
-    return esym().is_abs();
-
   return !is_imported && !get_frag() && !get_input_section() &&
          !get_output_section();
 }
