@@ -1,4 +1,6 @@
 #include "mold.h"
+
+#include "config.h"
 #include "blake3.h"
 
 #include <cctype>
@@ -503,7 +505,7 @@ void ShstrtabSection<E>::update_shdr(Context<E> &ctx) {
   i64 offset = 1;
 
   for (Chunk<E> *chunk : ctx.chunks) {
-    if (chunk->kind() != ChunkKind::HEADER && !chunk->name.empty()) {
+    if (!chunk->is_header() && !chunk->name.empty()) {
       auto [it, inserted] = map.insert({chunk->name, offset});
       chunk->shdr.sh_name = it->second;
       if (inserted)
@@ -520,7 +522,7 @@ void ShstrtabSection<E>::copy_buf(Context<E> &ctx) {
   base[0] = '\0';
 
   for (Chunk<E> *chunk : ctx.chunks)
-    if (chunk->kind() != ChunkKind::HEADER && !chunk->name.empty())
+    if (!chunk->is_header() && !chunk->name.empty())
       write_string(base + chunk->shdr.sh_name, chunk->name);
 }
 
@@ -859,36 +861,6 @@ void DynamicSection<E>::copy_buf(Context<E> &ctx) {
   write_vector(ctx.buf + this->shdr.sh_offset, contents);
 }
 
-template <typename E>
-OutputSection<E>::OutputSection(Context<E> &ctx, std::string_view name,
-                                u32 type, u64 flags) {
-  this->name = name;
-  this->shdr.sh_type = type;
-  this->shdr.sh_flags = flags & ~SHF_MERGE & ~SHF_STRINGS;
-
-  if (auto it = ctx.arg.section_align.find(name);
-      it != ctx.arg.section_align.end())
-    this->shdr.sh_addralign = it->second;
-
-  // PT_GNU_RELRO segment is a security mechanism to make more pages
-  // read-only than we could have done without it.
-  //
-  // Traditionally, sections are either read-only or read-write. If a
-  // section contains dynamic relocations, it must have been put into a
-  // read-write segment so that the program loader can mutate its
-  // contents in memory, even if no one will write to it at runtime.
-  //
-  // RELRO segment allows us to make such pages writable only when a
-  // program is being loaded. After that, the page becomes read-only.
-  //
-  // Some sections, such as .init, .fini, .got, .dynamic, contain
-  // dynamic relocations but doesn't have to be writable at runtime,
-  // so they are put into a RELRO segment.
-  this->is_relro = (name == ".toc" || name.ends_with(".rel.ro") ||
-                    type == SHT_INIT_ARRAY || type == SHT_FINI_ARRAY ||
-                    type == SHT_PREINIT_ARRAY || (flags & SHF_TLS));
-}
-
 template <typename E>
 void OutputSection<E>::copy_buf(Context<E> &ctx) {
   if (this->shdr.sh_type != SHT_NOBITS)
@@ -1651,6 +1623,7 @@ ElfSym<E> to_output_esym(Context<E> &ctx, Symbol<E> &sym, u32 st_name,
     esym.st_value = sym.get_addr(ctx);
   } else if (sym.file->is_dso || sym.esym().is_undef()) {
     esym.st_shndx = SHN_UNDEF;
+    esym.st_size = 0;
     if (sym.is_canonical)
       esym.st_value = sym.get_plt_addr(ctx);
   } else if (Chunk<E> *osec = sym.get_output_section()) {
@@ -2615,13 +2588,11 @@ static void compute_blake3(Context<E> &ctx, i64 offset) {
     u8 *end = (i == num_shards - 1) ? buf + filesize : begin + shard_size;
     blake3_hash(begin, end - begin, shards.data() + i * BLAKE3_OUT_LEN);
 
-#ifndef _WIN32
-    // We call munmap early for each chunk so that the last munmap
-    // gets cheaper. We assume that the .note.build-id section is
-    // at the beginning of an output file. This is an ugly performance
-    // hack, but we can save about 30 ms for a 2 GiB output.
+#ifdef HAVE_MADVISE
+    // Make the kernel page out the file contents we've just written
+    // so that subsequent close(2) call will become quicker.
     if (i > 0 && ctx.output_file->is_mmapped)
-      munmap(begin, end - begin);
+      madvise(begin, end - begin, MADV_DONTNEED);
 #endif
    });
 
@@ -2630,13 +2601,6 @@ static void compute_blake3(Context<E> &ctx, i64 offset) {
   u8 digest[BLAKE3_OUT_LEN];
   blake3_hash(shards.data(), shards.size(), digest);
   memcpy(buf + offset, digest, ctx.arg.build_id.size());
-
-#ifndef _WIN32
-  if (ctx.output_file->is_mmapped) {
-    munmap(buf, std::min(filesize, shard_size));
-    ctx.output_file->is_unmapped = true;
-  }
-#endif
 }
 
 template <typename E>
@@ -2785,11 +2749,11 @@ CompressedSection<E>::CompressedSection(Context<E> &ctx, Chunk<E> &chunk) {
   switch (ctx.arg.compress_debug_sections) {
   case COMPRESS_ZLIB:
     chdr.ch_type = ELFCOMPRESS_ZLIB;
-    compressed.reset(new ZlibCompressor(buf, chunk.shdr.sh_size));
+    compressor.reset(new ZlibCompressor(buf, chunk.shdr.sh_size));
     break;
   case COMPRESS_ZSTD:
     chdr.ch_type = ELFCOMPRESS_ZSTD;
-    compressed.reset(new ZstdCompressor(buf, chunk.shdr.sh_size));
+    compressor.reset(new ZstdCompressor(buf, chunk.shdr.sh_size));
     break;
   default:
     unreachable();
@@ -2801,7 +2765,7 @@ CompressedSection<E>::CompressedSection(Context<E> &ctx, Chunk<E> &chunk) {
   this->shdr = chunk.shdr;
   this->shdr.sh_flags |= SHF_COMPRESSED;
   this->shdr.sh_addralign = 1;
-  this->shdr.sh_size = sizeof(chdr) + compressed->compressed_size;
+  this->shdr.sh_size = sizeof(chdr) + compressor->compressed_size;
   this->shndx = chunk.shndx;
 
   // We don't need to keep the original data unless --gdb-index is given.
@@ -2815,7 +2779,7 @@ template <typename E>
 void CompressedSection<E>::copy_buf(Context<E> &ctx) {
   u8 *base = ctx.buf + this->shdr.sh_offset;
   memcpy(base, &chdr, sizeof(chdr));
-  compressed->write_to(base + sizeof(chdr));
+  compressor->write_to(base + sizeof(chdr));
 }
 
 template <typename E>
