@@ -19,10 +19,10 @@
 // image" in the PT_TLS segment. For each newly created thread including the
 // initial one, the runtime allocates a contiguous memory for an executable
 // and its depending shared libraries and copies template images there. That
-// per-thread memory is called the "TLS block". After allocating and
-// initializing a TLS block, the runtime sets a register to refer to the TLS
-// block, so that the thread-local variables are accessible relative to the
-// register.
+// per-thread memory is called the "initial TLS block". After allocating and
+// initializing the initial TLS block, the runtime sets a register to refer
+// to the initial TLS block, so that the thread-local variables are
+// accessible relative to the register.
 //
 // The register referring to the per-thread storage is called the Thread
 // Pointer (TP). TP is part of the thread's context. When the kernel
@@ -37,36 +37,36 @@
 // follows:
 //
 //  1. If we are creating an executable, we know the exact size of the TLS
-//     template image we are creating, and we know where the TP will be
-//     set to after the template is copied to the TLS block. Therefore,
+//     template image we are creating, and we know where the TP will be set
+//     to after the template is copied to the initial TLS block. Therefore,
 //     the TP-relative address of a TLV in the main executable is known at
 //     link-time. That means, computing a TLV's address can be as easy as
 //     `add %dst, %tp, <link-time constant>`.
 //
 //  2. If we are creating a shared library, we don't exactly know where
 //     its TLS template image will be copied to in terms of the
-//     TP-relative address, because we don't know how large is the main
+//     TP-relative address, because we don't know how large the main
 //     executable's and other libraries' TLS template images are. Only the
 //     runtime knows the exact TP-relative address.
 //
 //     We can solve the problem with an indirection. Specifically, for
-//     each TLV whose TP-relative address is only known at process startup
+//     each TLV whose TP-relative address is known only at process startup
 //     time, we create a GOT entry to store its TP-relative address. We
-//     also emit a dynamic relocation to let the runtime to fill the GOT
+//     then emit a dynamic relocation to let the runtime to fill the GOT
 //     entry with a TP-relative address.
 //
 //     Computing a TLV address in this scheme needs at least two machine
-//     instructions in most ISAs; first instruction loads a value from a
-//     GOT entry, and the second one adds the loaded value to TP.
+//     instructions in most ISAs; the first instruction loads a value from
+//     the GOT entry, and the second one adds the loaded value to TP.
 //
-//  3. Now, think about libraries that you dynamically load with dlopen.
-//     The TLS block for such library has to be allocated separately from
-//     the initial TLS block, so we now have two or more discontiguous
-//     TLS blocks. There's no easy formula to compute an address of a TLV
-//     in a separate TLS block.
+//  3. Now, think about libraries that are dynamically loaded with dlopen.
+//     The TLS block for such library may not be allocated next to the
+//     initial TLS block, so we can have two or more discontiguous TLS
+//     blocks. There's no easy formula to compute an address of a TLV in a
+//     separate TLS block.
 //
 //     The address of a TLV in a separate TLS block can be obtained by
-//     calling a libc-provided function, __tls_get_addr(). The function
+//     calling the libc-provided function, __tls_get_addr(). The function
 //     takes two arguments; a module ID to identify the ELF file and the
 //     TLV's offset within the ELF file's TLS template image. Accessing a
 //     TLV is sometimes compiled to a function call! The module ID and the
@@ -88,7 +88,6 @@
 // or DTP. We can then compute TLVs' addresses by adding their DTP-relative
 // addresses to DTP. This access model is called the Local Dynamic.
 //
-//
 // === TLS Descriptor access model ===
 //
 // As described above, there are arguably too many different TLS access
@@ -151,57 +150,58 @@ u64 get_tp_addr(Context<E> &ctx) {
   if (!phdr)
     return 0;
 
-  // On x86, SPARC and s390x, TP (%gs on i386, %fs on x86-64, %g7 on SPARC
-  // and %a0/%a1 on s390x) refers past the end of the TLS block for
-  // historical reasons. TLVs are accessed with negative offsets from TP.
-  if constexpr (is_x86<E> || is_sparc<E> || is_s390x<E>)
+  if constexpr (is_x86<E> || is_sparc<E> || is_s390x<E>) {
+    // On x86, SPARC and s390x, TP (%gs on i386, %fs on x86-64, %g7 on SPARC
+    // and %a0/%a1 on s390x) refers to past the end of the TLS block for
+    // historical reasons. TLVs are accessed with negative offsets from TP.
     return align_to(phdr->p_vaddr + phdr->p_memsz, phdr->p_align);
-
-  // On ARM, SH4 and Alpha, the runtime appends two words at the beginning
-  // of TLV template image when copying TLVs to the TLS block, so we need
-  // to offset it.
-  if constexpr (is_arm<E> || is_sh4<E> || is_alpha<E>)
+  } else if constexpr (is_arm<E> || is_sh4<E> || is_alpha<E>) {
+    // On ARM, SH4 and Alpha, the runtime appends two words at the beginning
+    // of TLV template image when copying TLVs to the TLS block, so we need
+    // to offset it.
     return align_down(phdr->p_vaddr - sizeof(Word<E>) * 2, phdr->p_align);
-
-  // On PPC and m68k, TP is 0x7000 (28 KiB) past the beginning of the TLV
-  // block to maximize the addressable range for load/store instructions
-  // with 16-bits signed immediates. It's not exactly 0x8000 (32 KiB) off
-  // because there's a small implementation-defined piece of data before
-  // the TLV block, and the runtime wants to access them efficiently too.
-  if constexpr (is_ppc<E> || is_m68k<E>)
+  } else if constexpr (is_ppc<E> || is_m68k<E>) {
+    // On PowerPC and m68k, TP is 0x7000 (28 KiB) past the beginning
+    // of the TLV block to maximize the addressable range of load/store
+    // instructions with 16-bits signed immediates. It's not exactly 0x8000
+    // (32 KiB) off because there's a small implementation-defined piece of
+    // data before the initial TLV block, and the runtime wants to access
+    // them efficiently too.
     return phdr->p_vaddr + 0x7000;
-
-  // RISC-V just uses the beginning of the main executable's TLV block as
-  // TP. RISC-V load/store instructions usually take 12-bits signed
-  // immediates, so the beginning of TLV ± 2 KiB is accessible with a
-  // single load/store instruction.
-  assert(is_riscv<E>);
-  return phdr->p_vaddr;
+  } else {
+    // RISC-V and LoongArch just uses the beginning of the main executable's
+    // TLV block as TP. Their load/store instructions usually take 12-bits
+    // signed immediates, so the beginning of the TLS block ± 2 KiB is
+    // accessible with a single load/store instruction.
+    static_assert(is_riscv<E> || is_loongarch<E>);
+    return phdr->p_vaddr;
+  }
 }
 
-// Returns the address when __tls_get_addr would be called with offset 0.
+// Returns the address __tls_get_addr() would return if it's called
+// with offset 0.
 template <typename E>
 u64 get_dtp_addr(Context<E> &ctx) {
   ElfPhdr<E> *phdr = get_tls_segment(ctx);
   if (!phdr)
     return 0;
 
-  // On PPC64 and m68k, R_DTPOFF is resolved to the address 0x8000 (32
-  // KiB) past the start of the TLS block. The bias maximizes the
-  // accessible range for load/store instructions with 16-bits signed
-  // immediates. That is, if the offset were right at the beginning of
-  // the start of the TLS block, the half of addressible space (negative
-  // immediates) would have been wasted.
-  if constexpr (is_ppc<E> || is_m68k<E>)
+  if constexpr (is_ppc<E> || is_m68k<E>) {
+    // On PowerPC and m68k, R_DTPOFF is resolved to the address 0x8000
+    // (32 KiB) past the start of the TLS block. The bias maximizes the
+    // accessible range of load/store instructions with 16-bits signed
+    // immediates. That is, if the offset were right at the beginning of the
+    // start of the TLS block, the half of addressible space (negative
+    // immediates) would have been wasted.
     return phdr->p_vaddr + 0x8000;
-
-  // On RISC-V, the bias is 0x800 as the load/store instructions in the
-  // ISA usually have a 12-bit immediate.
-  if constexpr (is_riscv<E>)
+  } else if constexpr (is_riscv<E>) {
+    // On RISC-V, the bias is 0x800 as the load/store instructions in the
+    // ISA usually have a 12-bit immediate.
     return phdr->p_vaddr + 0x800;
-
-  // On other targets, DTP simply refers to the beginning of the TLS block.
-  return phdr->p_vaddr;
+  } else {
+    // On other targets, DTP simply refers to the beginning of the TLS block.
+    return phdr->p_vaddr;
+  }
 }
 
 using E = MOLD_TARGET;
