@@ -6,4 +6,5 @@ int main() {}
 EOF
 
 $CC -B. -o $t/exe $t/a.o
-readelf -p .comment $t/exe | grep -q 'mold'
+readelf -p .comment $t/exe | grep -q '[ms]old'
+readelf -SW $t/exe | grep -Eq '\.comment.*\bMS\b'
