@@ -1,16 +1,5 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
 cat <<EOF | $CC -fcommon -xc -c -o $t/a.o -
 #include <stdio.h>
@@ -55,5 +44,3 @@ ar rcs $t/f.a $t/b.o $t/f.o
 
 $CC -B. -o $t/exe $t/a.o $t/f.a
 $QEMU $t/exe | grep -q '5 0 7 2'
-
-echo OK
