@@ -1,24 +1,12 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
-[ $MACHINE = $(uname -m) ] || { echo skipped; exit; }
-[ $MACHINE = riscv64 -o $MACHINE = riscv32 ] && { echo skipped; exit; }
+[ "$QEMU" = '' ] || skip
+[ $MACHINE = riscv64 -o $MACHINE = riscv32 -o $MACHINE = sparc64 ] && skip
 
-which gdb >& /dev/null || { echo skipped; exit; }
+command -v gdb >& /dev/null || skip
 
-echo 'int main() {}' | $CC -o /dev/null -xc -gdwarf-2 -g - >& /dev/null ||
-  { echo skipped; exit; }
+echo 'int main() {}' | $CC -o /dev/null -xc -gdwarf-2 -g - >& /dev/null || skip
 
 cat <<EOF | $CC -c -o $t/a.o -fPIC -g -ggnu-pubnames -gdwarf-2 -xc - -ffunction-sections
 void hello2();
@@ -44,7 +32,7 @@ void hello2() {
 EOF
 
 $CC -B. -shared -o $t/c.so $t/a.o $t/b.o -Wl,--gdb-index
-readelf -WS $t/c.so 2> /dev/null | fgrep -q .gdb_index
+readelf -WS $t/c.so 2> /dev/null | grep -Fq .gdb_index
 
 cat <<EOF | $CC -c -o $t/d.o -fPIC -g -ggnu-pubnames -gdwarf-2 -xc - -gz
 void greet();
@@ -55,7 +43,7 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe $t/c.so $t/d.o -Wl,--gdb-index
-readelf -WS $t/exe 2> /dev/null | fgrep -q .gdb_index
+readelf -WS $t/exe 2> /dev/null | grep -Fq .gdb_index
 
 $QEMU $t/exe | grep -q 'Hello world'
 
@@ -66,5 +54,3 @@ grep -q 'hello2 () at .*<stdin>:7' $t/log
 grep -q 'hello () at .*<stdin>:4' $t/log
 grep -q 'greet () at .*<stdin>:8' $t/log
 grep -q 'main () at .*<stdin>:4' $t/log
-
-echo OK
