@@ -33,5 +33,8 @@ int main() {
 }
 EOF
 
-$CC -B. -o $t/exe $t/a.o $t/b.o $t/c.o
-$QEMU $t/exe | grep -q '^3$'
+$CC -B. -o $t/exe1 $t/a.o $t/b.o $t/c.o -pie
+$QEMU $t/exe1 | grep -q '^3$'
+
+$CC -B. -o $t/exe2 $t/a.o $t/b.o $t/c.o -no-pie
+$QEMU $t/exe2 | grep -q '^3$'
