@@ -1,16 +1,5 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
 cat <<EOF | $CC -fPIC -c -o $t/a.o -xc -
 extern int foo;
@@ -29,5 +18,3 @@ EOF
 
 $CC -B. -shared -o $t/e.so $t/a.o $t/d.o -Wl,-strip-all
 readelf --symbols $t/e.so | grep -Eq 'PROTECTED\b.*\bfoo\b'
-
-echo OK
