@@ -4,7 +4,8 @@
 # OneTBB isn't tsan-clean
 nm mold | grep -q '__tsan_init' && skip
 
-# Ubuntu 22.04 SH4 GCC is broken
+# Ubuntu 22.04 GCC is broken
+[ $MACHINE = m68k ] && skip
 [ $MACHINE = sh4 ] && skip
 
 cat <<EOF | $CXX -c -o $t/a.o -xc++ -
