@@ -1,16 +1,5 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
 cat <<EOF | $CC -c -xc -o $t/a.o -
 #include <stdio.h>
@@ -33,5 +22,3 @@ $CC -B. -o $t/exe3 $t/a.o -Wl,-z,norelro
 $QEMU $t/exe3 | grep -q 'Hello world'
 readelf --segments -W $t/exe3 > $t/log3
 ! grep -q 'GNU_RELRO ' $t/log3 || false
-
-echo OK
