@@ -12,23 +12,23 @@ cat <<'EOF' | $CC -c -o $t/a.o -x assembler -
 val1:
 .ascii "Hello \0"
 
-.section .rodata.str4.4,"aMS",@progbits,4
-.align 4
+.section .rodata.foo,"aMS",@progbits,4
+.p2align 2
 val2:
 .ascii "world   \0\0\0\0"
 
-.section .rodata.foo,"aMS",@progbits,1
+.section .rodata.x,"aMS",@progbits,1
 val3:
 .ascii "foobar\0"
 
 .section .rodata.cst8,"aM",@progbits,8
-.align 8
+.p2align 3
 val4:
 .ascii "abcdefgh"
 
-.section .rodatabar,"aMS",@progbits,1
+.section .rodatabaz,"aMS",@progbits,1
 val5:
-.ascii "bar\0"
+.ascii "baz\0"
 EOF
 
 cat <<'EOF' | $CC -c -o $t/b.o -xc -
@@ -43,8 +43,8 @@ EOF
 
 $CC -B. -o $t/exe $t/a.o $t/b.o
 
-readelf -p .rodata.str $t/exe | grep -q Hello
-readelf -p .rodata.str $t/exe | grep -q world
-readelf -p .rodata.str $t/exe | grep -q foobar
-readelf -p .rodata.cst $t/exe | grep -q abcdefgh
-readelf -p .rodatabar $t/exe | grep -q bar
+readelf -p .rodata.str1.1 $t/exe | grep -q Hello
+readelf -p .rodata.str4.4 $t/exe | grep -q world
+readelf -p .rodata.str1.1 $t/exe | grep -q foobar
+readelf -p .rodata.cst8 $t/exe | grep -q abcdefgh
+readelf -p .rodatabaz $t/exe | grep -q baz
