@@ -15,15 +15,13 @@ int main() { printf("foo=%p\n", &foo); }
 EOF
 
 $CC -fPIC -c -o $t/d.o $t/c.c
-$CC -B. -o $t/exe1 -pie $t/d.o $t/b.so
-$QEMU $t/exe1 | grep -q 'foo=0x3'
 
-nm -D $t/exe1 > $t/log1
-! grep -q foo $t/log1 || false
+# This test fails with older glibc
+$CC -o $t/exe1 -pie $t/d.o $t/b.so 2> /dev/null || skip
+$QEMU $t/exe1 | grep -q 'foo=0x3' || skip
 
-$CC -fPIC -c -o $t/e.o $t/c.c
-$CC -B. -o $t/exe2 -no-pie $t/e.o $t/b.so
+$CC -B. -o $t/exe2 -pie $t/d.o $t/b.so
 $QEMU $t/exe2 | grep -q 'foo=0x3'
 
-nm -D $t/exe2 > $t/log2
-! grep -q foo $t/log2 || false
+$CC -B. -o $t/exe3 -no-pie $t/d.o $t/b.so
+$QEMU $t/exe3 | grep -q 'foo=0x3'
