@@ -1,16 +1,5 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
 cat <<'EOF' | $CC -fPIC -c -o $t/a.o -xc -
 void fn2();
@@ -43,5 +32,3 @@ EOF
 $CC -B. -o $t/exe $t/c.o $t/b.so
 $QEMU $t/exe | grep -q hello
 ! readelf --symbols $t/exe | grep -q fn3 || false
-
-echo OK
