@@ -10,5 +10,8 @@ int main() {
 }
 EOF
 
-$CC -B. -o $t/exe2 $t/a.o -static-pie
+$CC -B. -o $t/exe1 $t/a.o -static-pie
+$QEMU $t/exe1 | grep -q 'Hello world'
+
+$CC -B. -o $t/exe2 $t/a.o -static-pie -Wl,--no-relax
 $QEMU $t/exe2 | grep -q 'Hello world'
