@@ -1,22 +1,8 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
-[ $MACHINE = x86_64 ] || { echo skipped; exit; }
-
-cat <<EOF | $CC -c -o $t/a.o -x assembler -
-.section foo,"a",@progbits
-.ascii "section foo"
+cat <<EOF | $CC -c -o $t/a.o -xc -
+__attribute__((section("foo"))) char bar[] = "section foo";
 EOF
 
 # Test synthetic symbols
@@ -106,5 +92,3 @@ grep -q '^edata=foo$' $t/log
 grep -q '^__ehdr_start=0x40000$' $t/log
 grep -q '^__executable_start=0x40000$' $t/log
 grep -q '^section foo$' $t/log
-
-echo OK
