@@ -1,15 +1,9 @@
 #!/bin/bash
 . $(dirname $0)/common.inc
 
-if [ $MACHINE = x86_64 -o $MACHINE = arm ]; then
-  dialect=gnu2
-elif [ $MACHINE = aarch64 ]; then
-  dialect=desc
-else
-  skip
-fi
+supports_tlsdesc || skip
 
-cat <<EOF | $GCC -fPIC -mtls-dialect=$dialect -c -o $t/a.o -xc -
+cat <<EOF | $GCC -fPIC -c -o $t/a.o -xc - $tlsdesc_opt
 #include <stdio.h>
 
 extern _Thread_local int foo;
@@ -21,7 +15,7 @@ int main() {
 }
 EOF
 
-cat <<EOF | $GCC -fPIC -mtls-dialect=$dialect -shared -o $t/b.so -xc -
+cat <<EOF | $GCC -fPIC -shared -o $t/b.so -xc - $tlsdesc_opt
 _Thread_local int foo = 5;
 _Thread_local int bar;
 EOF
