@@ -1,16 +1,7 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
+
+test_cflags -static || skip
 
 cat <<EOF | $CC -o $t/a.o -c -x assembler -
 .globl _start
@@ -44,5 +35,3 @@ grep -q foo $t/log
 readelf --symbols $t/exe > $t/log
 grep -q foo $t/log
 grep -q bar $t/log
-
-echo OK
