@@ -20,6 +20,6 @@ EOF
 $CC -B. -shared -Wl,--version-script=$t/a.ver -o $t/c.so $t/b.o
 
 readelf --dyn-syms $t/c.so > $t/log
-grep -q ' xyz$' $t/log
-grep -q ' foo_bar$' $t/log
+grep -q ' xyz' $t/log
+grep -q ' foo_bar' $t/log
 ! grep -q ' foo$' $t/log || false
