@@ -16,5 +16,5 @@ EOF
 $CC -B. -shared -Wl,--version-script=$t/a.ver -o $t/c.so $t/b.o
 
 readelf --dyn-syms $t/c.so > $t/log
-grep -q ' foobar$' $t/log
+grep -q ' foobar' $t/log
 ! grep -q ' foo$' $t/log || false
