@@ -23,9 +23,9 @@ EOF
 $CC -B. -shared -Wl,--version-script=$t/a.ver -o $t/c.so $t/b.o
 
 readelf --dyn-syms $t/c.so > $t/log
-grep -q ' xyz$' $t/log
-! grep -q ' foobarzx$' $t/log || false
-grep -q ' foobarcx$' $t/log
-grep -q ' foo123bar456bx$' $t/log
-! grep -q ' foo123bar456c$' $t/log || false
-! grep -q ' foo123bar456x$' $t/log || false
+grep -q ' xyz' $t/log
+! grep -q ' foobarzx' $t/log || false
+grep -q ' foobarcx' $t/log
+grep -q ' foo123bar456bx' $t/log
+! grep -q ' foo123bar456c' $t/log || false
+! grep -q ' foo123bar456x' $t/log || false
