@@ -20,7 +20,7 @@ EOF
 $CC -B. -shared -Wl,--version-script=$t/a.ver -o $t/c.so $t/b.o
 
 readelf --dyn-syms $t/c.so > $t/log
-grep -q ' azZ$' $t/log
-grep -q ' czZ$' $t/log
-! grep -q ' azC$' $t/log || false
-! grep -q ' aaZ$' $t/log || false
+grep -q ' azZ' $t/log
+grep -q ' czZ' $t/log
+! grep -q ' azC' $t/log || false
+! grep -q ' aaZ' $t/log || false
