@@ -1,16 +1,5 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
 cat <<EOF > $t/a.ver
 ver1 {
@@ -50,8 +39,6 @@ $CC -B. -o $t/exe $t/c.o $t/b.so
 $QEMU $t/exe
 
 readelf --dyn-syms $t/exe > $t/log
-fgrep -q 'foo@ver1' $t/log
-fgrep -q 'bar@ver2' $t/log
-fgrep -q 'baz@ver3' $t/log
-
-echo OK
+grep -Fq 'foo@ver1' $t/log
+grep -Fq 'bar@ver2' $t/log
+grep -Fq 'baz@ver3' $t/log
