@@ -1,16 +1,5 @@
 #!/bin/bash
-export LC_ALL=C
-set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
-testname=$(basename "$0" .sh)
-echo -n "Testing $testname ... "
-t=out/test/elf/$MACHINE/$testname
-mkdir -p $t
+. $(dirname $0)/common.inc
 
 cat <<EOF | $CC -o $t/a.o -c -x assembler -
 .globl _start
@@ -41,5 +30,3 @@ $CC -B. -nostdlib -o $t/exe $t/a.o -Wl,--whole-archive \
 readelf --symbols $t/exe > $t/readelf
 ! grep -q fn1 $t/readelf || false
 ! grep -q fn2 $t/readelf || false
-
-echo OK
