@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2021 Intel Corporation
+    Copyright (c) 2005-2022 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@
 #include <algorithm>
 #include <cstring>
 #include <chrono>
+#include <unordered_set>
 
 #if HARNESS_TBBMALLOC_THREAD_SHUTDOWN && __TBB_SOURCE_DIRECTLY_INCLUDED && (_WIN32 || _WIN64)
 #include "../../src/tbbmalloc/tbbmalloc_internal_api.h"
@@ -437,6 +438,43 @@ concept well_formed_instantiation = requires {
 };
 #endif // __TBB_CPP20_CONCEPTS_PRESENT
 
+class LifeTrackableObject {
+    using set_type = std::unordered_set<const LifeTrackableObject*>;
+    static set_type alive_objects;
+public:
+    LifeTrackableObject() {
+        alive_objects.insert(this);
+    }
+
+    LifeTrackableObject(const LifeTrackableObject&) {
+        alive_objects.insert(this);
+    }
+
+    LifeTrackableObject(LifeTrackableObject&&) {
+        alive_objects.insert(this);
+    }
+
+    LifeTrackableObject& operator=(const LifeTrackableObject&) = default;
+    LifeTrackableObject& operator=(LifeTrackableObject&&) = default;
+
+    ~LifeTrackableObject() {
+        alive_objects.erase(this);
+    }
+
+    static bool is_alive(const LifeTrackableObject& object) {
+        return is_alive(&object);
+    }
+
+    static bool is_alive(const LifeTrackableObject* object) {
+        return alive_objects.find(object) != alive_objects.end();
+    }
+
+    static const set_type& set() {
+        return alive_objects;
+    }
+};
+std::unordered_set<const LifeTrackableObject*> LifeTrackableObject::alive_objects{};
+
 } // namespace utils
 
 #endif // __TBB_test_common_utils_H
